const std = @import("std");

pub fn build(b: *std.Build) !void {
    const target = b.standardTargetOptions(.{});
    const optimize = b.standardOptimizeOption(.{});

    // from riverwm docs generation
    const man_pages = b.option(
        bool,
        "man-pages",
        "Set to true to build man pages. Requires scdoc. Defaults to true if scdoc is found.",
    ) orelse scdoc_found: {
        _ = b.findProgram(&.{"scdoc"}, &.{}) catch |err| switch (err) {
            error.FileNotFound => break :scdoc_found false,
            else => return err,
        };
        break :scdoc_found true;
    };
    if (man_pages) {
        inline for (.{ "honeybee", "honeyctl" }) |page| {
            const scdoc = b.addSystemCommand(&.{ "/bin/sh", "-c", "scdoc < docs/" ++ page ++ ".1.scd" });
            // This makes the caching work for the Workaround, and the extra argument is ignored by /bin/sh.
            scdoc.addFileArg(b.path("docs/" ++ page ++ ".1.scd"));

            const stdout = scdoc.captureStdOut();
            b.getInstallStep().dependOn(&b.addInstallFile(stdout, "share/man/man1/" ++ page ++ ".1").step);
        }
    }

    // module build
    const mod = b.addModule("libhoneybee", .{
        .root_source_file = b.path("src/lib/root.zig")
    });

    // daemon build
    const exe = b.addExecutable(.{
        .name = "honeybee",
        .root_module = b.createModule(.{
            .root_source_file = b.path("src/honeybee/main.zig"),
            .imports = &.{
                .{ .name = "libhoneybee", .module = mod},
            },
            .target = target,
            .optimize = optimize,
        }),
    });
    b.installArtifact(exe);
    exe.linkLibC();
    exe.addIncludePath(b.path("./src/honeybee/pulse_c"));
    exe.addCSourceFile(.{ .file = b.path("src/honeybee/pulse_c/audio_be.c") });
    exe.linkSystemLibrary("snikket");
    exe.linkSystemLibrary("libpulse-simple");

    // add cli tool
    const exe_c = b.addExecutable(.{
        .name = "honeyctl",
        .root_module = b.createModule(.{
            .root_source_file = b.path("src/honeyctl/main.zig"),
            .imports = &.{
                .{ .name = "libhoneybee", .module = mod},
            },
            .target = target,
            .optimize = optimize,
        }),
    });
    b.installArtifact(exe_c);

    // run command
    const run_cmd = b.addRunArtifact(exe);
    run_cmd.step.dependOn(b.getInstallStep());
    if (b.args) |args| {
        run_cmd.addArgs(args);
    }
    const run_step = b.step("run", "Run the app");
    run_step.dependOn(&run_cmd.step);
}
