const std = @import("std");
const config_common = @import("libhoneybee").config;
const max_config_size = 512;

const config = struct {
    username: [:0]const u8,
    password_eval: [:0]const u8,
};

const config_full = struct {
    password: [:0]const u8,
    json: std.json.Parsed(config),
    env_variables: config_common.env_variables,
    allocator: std.mem.Allocator,

    pub fn free(self: *const config_full) void {
        self.allocator.free(self.password);
        self.json.deinit();
        self.env_variables.free();
    }
};

pub fn get_config(allocator: std.mem.Allocator) !config_full {
    const env_vars = try config_common.get_env_variables(allocator);
    const hb_cfg_path: []const u8 = try std.fs.path.join(allocator, &[_][]const u8{ env_vars.xdg_config_home, "honeybee", "config.json" });
    defer allocator.free(hb_cfg_path);

    const data = try std.fs.cwd().readFileAlloc(allocator, hb_cfg_path, max_config_size);
    defer allocator.free(data);

    // TODO: improve errors to user when unknown field in config
    const parsed = try std.json.parseFromSlice(config, allocator, data, .{ .allocate = .alloc_always });

    if (parsed.value.password_eval.len == 0) {
        @panic("password_eval is not set - you need to provide a command that gives honeybee the password");
    }

    const password_eval_result = try std.process.Child.run(.{
        .allocator = allocator,
        .argv = &[_][]const u8{ "sh", "-c", parsed.value.password_eval },
    });
    defer allocator.free(password_eval_result.stdout);
    defer allocator.free(password_eval_result.stderr);

    if (password_eval_result.stdout.len == 0) {
        @panic("password_eval command returned an empty string to stdout");
    }

    const password = try allocator.allocSentinel(u8, password_eval_result.stdout.len, 0);
    @memcpy(password, password_eval_result.stdout);

    return config_full{
        .json = parsed,
        .password = password,
        .env_variables = env_vars,
        .allocator = allocator,
    };
}
