const std = @import("std");
const state = @import("state.zig");
const c_time = @cImport({
    @cInclude("time.h");
});

pub const FileLog = struct {
    state_path: []const u8,
    allocator: std.mem.Allocator,

    pub fn create_path(self: *const FileLog) void {
        // create path required for state file
        // Should only be run once
        const hb_runtime_path: []const u8 = std.fs.path.join(self.allocator, &[_][]const u8{ self.runtime_dir, "honeybee" }) catch unreachable;
        defer self.allocator.free(hb_runtime_path);

        std.log.debug("Making dir {s}", .{hb_runtime_path});
        std.fs.makeDirAbsolute(hb_runtime_path) catch |err| switch (err) {
            std.posix.MakeDirError.PathAlreadyExists => return,
            else => unreachable,
        };
    }

    pub fn update_state_file(self: *const FileLog, status: state.HookType, jid: []u8) void {
        const event_name = @tagName(status);

        std.log.debug("Updating {s} to: {s}", .{ self.state_path, event_name });
        const file = std.fs.createFileAbsolute(self.state_path, .{ .truncate = true, .exclusive = false }) catch unreachable;
        defer file.close();

        try file.seekFromEnd(0);

        // TODO: only do this if not missed call
        const time_stamp = get_isotime();
        defer self.allocator.free(time_stamp);

        _ = file.write("{s}\t{s}\t{s}", .{ @tagName(status), jid, time_stamp }) catch unreachable;
    }
};

fn get_isotime(allocator: std.mem.Allocator) []const u8 {
    var current: [*c]c_time.struct_tm = null;
    var time: c_time.time_t = c_time.time(null);
    const buffer_size = 64;
    const buffer = allocator.alloc(u8, buffer_size) catch unreachable;
    current = c_time.localtime(&time);

    _ = c_time.strftime(@ptrCast(buffer), buffer_size, "%Y-%m-%dT%H:%M:%S%z", current);
    return buffer;
}
