const std = @import("std");

// Setup configuration related to environment variables only
pub const env_variables = struct {
    xdg_config_home: []const u8,
    xdg_runtime_dir: []const u8,
    state_path: []const u8,
    allocator: std.mem.Allocator,

    pub fn free(self: *const env_variables) void {
        self.allocator.free(self.state_path);
    }
};

pub fn get_env_variables(allocator: std.mem.Allocator) !env_variables {
    // https://specifications.freedesktop.org/basedir-spec/latest/#variables
    const home_dir: []const u8 = std.posix.getenv("HOME") orelse unreachable;
    const default_config_path: []const u8 = try std.fs.path.join(allocator, &[_][]const u8{ home_dir, ".config" });
    defer allocator.free(default_config_path);
    const xdg_config_home: []const u8 = std.posix.getenv("XDG_CONFIG_HOME") orelse default_config_path;

    const xdg_runtime_dir: []const u8 = std.posix.getenv("XDG_RUNTIME_DIR") orelse unreachable;
    const hb_state_path: []const u8 = try std.fs.path.join(allocator, &[_][]const u8{ xdg_runtime_dir, "honeybee", "state" });

    return env_variables{
        .allocator = allocator,
        .xdg_config_home = xdg_config_home,
        .xdg_runtime_dir = xdg_runtime_dir,
        .state_path = hb_state_path,
    };
}
